@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: imp/compose.php,v 2.369.2.56 2003/03/08 01:15:32 slusarz Exp $
+ * $Horde: imp/compose.php,v 2.369.2.61 2003/07/08 19:40:20 slusarz Exp $
  *
  * Copyright 1999-2003 Charles J. Hagenbuch <chuck@horde.org>
  * Copyright 1999-2003 Jon Parise <jon@horde.org>
@@ -185,7 +185,7 @@ function expandAddresses($addrString, $full = false)
                     $arr[$i] = $res[0]['name'] . ': ' . $res[0]['email'] . ';';
                 } else {
                     list($mbox, $host) = explode('@', $res[0]['email']);
-                    $arr[$i] = imap_rfc822_write_address($mbox, $host, $res[0]['name']);
+                    $arr[$i] = IMP::rfc822WriteAddress($mbox, $host, $res[0]['name']);
                 }
             } else {
                 $arr[$i] = $res[0]['name'];
@@ -200,7 +200,7 @@ function expandAddresses($addrString, $full = false)
                     } else {
                         $mbox_host = explode('@', $one_res['email']);
                         if (isset($mbox_host[1])) {
-                            $arr[$i][] = imap_rfc822_write_address($mbox_host[0], $mbox_host[1], $one_res['name']);
+                            $arr[$i][] = IMP::rfc822WriteAddress($mbox_host[0], $mbox_host[1], $one_res['name']);
                         }
                     }
                 } else {
@@ -353,7 +353,8 @@ function addMimeParts(&$message)
 
             /* Base64-encode all attachments to preserve linebreaks,
                ensure transportability, etc. */
-            $contents = chunk_split(base64_encode($contents), 76, "\n");
+            $encoded_contents = base64_encode($contents);
+            $contents = chunk_split($encoded_contents, 76, "\n");
             $part->setTransferEncoding('base64');
             $part->setContents($contents);
             $message->addPart($part);
@@ -381,6 +382,18 @@ function attachFiles($structure, $ref)
     }
 }
 
+/**
+ * Retrieves and wraps the submitted message text.
+ */
+function getMessage()
+{
+    global $prefs;
+
+    $msg = Horde::getFormData('message', '');
+    $msg = Text::wrap($msg, $prefs->getValue('wrap_width'), "\n", Lang::getCharset());
+    return $msg;
+}
+
 
 define('IMP_BASE', dirname(__FILE__));
 
@@ -673,7 +686,7 @@ switch ($actionID) {
          $recipients = $f_to = format_addresses($f_to);
 
          /* Check for 8-bit characters in addresses. */
-         foreach (IMP::bareAddress($recips, true) as $val) {
+         foreach (IMP::bareAddress($recipients, true) as $val) {
              if (MIME::is8bit($val)) {
                  Horde::raiseMessage(_("Invalid character in e-mail address."), HORDE_ERROR);
                  $get_sig = false;
@@ -741,7 +754,7 @@ switch ($actionID) {
      break;
 
  case SEND_MESSAGE:
-     $message = Horde::getFormData('message', '');
+     $message = getMessage();
      $recips = null;
      if ($f_to = Horde::getFormData('to', getAddressList('to'))) {
          $recips = $f_to;
@@ -927,7 +940,7 @@ switch ($actionID) {
      $drafts_folder = $prefs->getValue('drafts_folder');
      if (!empty($drafts_folder)) {
          $drafts_folder = IMP::addPreambleString($drafts_folder);
-         if (($message = Horde::getFormData('message'))) {
+         if ($message = getMessage()) {
              $message = str_replace("\r\n", "\n", $message);
 
              include_once HORDE_BASE . '/lib/MIME/Message.php';
@@ -1167,7 +1180,7 @@ if (Horde::getFormData('from') || $prefs->isLocked('default_identity')) {
 }
 
 /* Grab any data that we were supplied with. */
-if (empty($msg)) $msg = "\n" . Horde::getFormData('message', '');
+if (empty($msg)) $msg = "\n" . getMessage();
 if (empty($to)) $to = Horde::getFormData('to', getAddressList('to'));
 if (empty($cc)) $cc = Horde::getFormData('cc', getAddressList('cc'));
 if (empty($bcc)) $bcc = Horde::getFormData('bcc', getAddressList('bcc'));
