@@ -1,6 +1,6 @@
 <?php
 /*
- * $Horde: imp/lib/Folder.php,v 1.53.2.12 2003/01/16 22:29:05 jan Exp $
+ * $Horde: imp/lib/Folder.php,v 1.53.2.14 2003/06/14 14:16:32 chuck Exp $
  *
  * Copyright 2000-2003 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 2000-2003 Jon Parise <jon@csh.rit.edu>
@@ -15,7 +15,7 @@
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@csh.rit.edu>
- * @version $Revision: 1.53.2.12 $
+ * @version $Revision: 1.53.2.14 $
  * @since   IMP 2.3
  * @package imp
  */
@@ -153,12 +153,13 @@ class IMP_Folder {
                         if (substr($label, 0, strlen($namespace)) == $namespace) {
                             $label = substr($label, strlen($namespace));
                         }
+                        $stripped = str_replace($label, '', $value);
                         $parts = explode($delimiter, $label);
                         $subfolder = array_pop($parts);
                         $parent = implode($delimiter, $parts);
                         if (!empty($parent) &&
                             ($parent != 'INBOX') &&
-                            !isset($list[$prefix . $namespace . $parent]) &&
+                            !isset($list[$stripped . $parent]) &&
                             !isset($parents[$parent])) {
                             $parents[$parent] = 1;
                             $plabel = str_repeat(' ', 4 * (count($parts) - 1)) . IMP::utf7Decode($parts[count($parts) - 1]) . $delimiter;
@@ -265,7 +266,7 @@ class IMP_Folder {
         foreach ($folder_array as $folder) {
             if (!imap_deletemailbox($stream, $server . $folder)) {
                 Horde::raiseMessage(sprintf(_("The folder \"%s\" was not deleted. This is what the server said"), IMP::displayFolder($folder)) .
-                                    ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
+                                    imap_last_error(), HORDE_ERROR);
                 $return_value = false;
             } else {
                 if ($subscribe && in_array($server . $folder, $subscribed_folders) &&
@@ -311,7 +312,7 @@ class IMP_Folder {
         /* Attempt to create the mailbox */
         if (!imap_createmailbox($stream, IMP::serverString() . $folder)) {
             Horde::raiseMessage(sprintf(_("The folder \"%s\" was not created. This is what the server said"), IMP::displayFolder($folder)) .
-                            ': <i>' . imap_last_error() . '</i>', HORDE_ERROR);
+                            imap_last_error(), HORDE_ERROR);
             return false;
         }
 
@@ -377,7 +378,7 @@ class IMP_Folder {
             return true;
         } else {
             Horde::raiseMessage(sprintf(_("Renaming \"%s\" to \"%s\" failed. This is what the server said"), IMP::displayFolder($old), IMP::displayFolder($new)) .
-                                ' : <i>' . imap_last_error() . '</i>', HORDE_ERROR);
+                                imap_last_error(), HORDE_ERROR);
             return false;
         }
     }
@@ -401,7 +402,7 @@ class IMP_Folder {
                 if ($folder != ' ') {
                     if (!imap_subscribe($stream, IMP::serverString() . $folder)) {
                         Horde::raiseMessage(sprintf(_("You were not subscribed to \"%s\". Here is what the server said"), IMP::displayFolder($folder)) .
-                                            ' :<i>' . imap_last_error() . '</i>', HORDE_ERROR);
+                                            imap_last_error(), HORDE_ERROR);
                         $return_value = false;
                     } else {
                         Horde::raiseMessage(sprintf(_("You were successfully subscribed to \"%s\""), IMP::displayFolder($folder)), HORDE_SUCCESS);
@@ -435,7 +436,7 @@ class IMP_Folder {
                 if ($folder != ' ') {
                     if (!imap_unsubscribe($stream, IMP::serverString() . $folder)) {
                         Horde::raiseMessage(sprintf(_("You were not unsubscribed from \"%s\". Here is what the server said"), IMP::displayFolder($folder)) .
-                                        ' :<i>' . imap_last_error() . '</i>', HORDE_ERROR);
+                                        imap_last_error(), HORDE_ERROR);
                         $return_value = false;
                     } else {
                         Horde::raiseMessage(sprintf(_("You were successfully unsubscribed from \"%s\""), IMP::displayFolder($folder)), HORDE_SUCCESS);
@@ -507,7 +508,7 @@ class IMP_Folder {
         static $delimiter;
 
         if (!isset($delimiter)) {
-            $box = imap_getmailboxes($stream, IMP::serverString(), IMP::addPreambleString('%'));
+            $box = imap_getmailboxes($stream, IMP::serverString(), 'INBOX');
             $delimiter = !empty($box[0]->delimiter) ? $box[0]->delimiter : $default;
         }
 
