@@ -5,7 +5,7 @@ require_once IMP_BASE . '/lib/constants.php';
 /**
  * IMP Base Class.
  *
- * $Horde: imp/lib/IMP.php,v 1.198.2.51 2003/02/12 23:08:32 jan Exp $
+ * $Horde: imp/lib/IMP.php,v 1.198.2.58 2003/08/21 01:54:07 chuck Exp $
  *
  * Copyright 1999-2003 Chuck Hagenbuch <chuck@horde.org>
  * Copyright 1999-2003 Jon Parise <jon@horde.org>
@@ -15,7 +15,7 @@ require_once IMP_BASE . '/lib/constants.php';
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jon Parise <jon@horde.org>
- * @version $Revision: 1.198.2.51 $
+ * @version $Revision: 1.198.2.58 $
  * @since   IMP 2.3.5
  * @package imp
  */
@@ -34,7 +34,7 @@ class IMP {
      */
     function createSession()
     {
-        global $imp, $conf, $registry;
+        global $conf, $registry;
 
         if (!isset($_POST['imapuser'])
             || !isset($_POST['pass'])
@@ -42,8 +42,8 @@ class IMP {
             return 'failed';
         }
 
-        $imp = array();
-        $imp['user'] = trim(Horde::getFormData('imapuser'));
+        $_SESSION['imp'] = array();
+        $_SESSION['imp']['user'] = trim(Horde::getFormData('imapuser'));
 
         /* Define these now so they're available to the vinfo
            hooks. */
@@ -53,39 +53,39 @@ class IMP {
         /* Run the username through virtualhost expansion functions if
            necessary. */
         if (!empty($conf['hooks']['vinfo']) && function_exists($conf['hooks']['vinfo'])) {
-            $imp['user'] = call_user_func($conf['hooks']['vinfo']);
+            $_SESSION['imp']['user'] = call_user_func($conf['hooks']['vinfo']);
         }
 
-        $imp['pass'] = Secret::write(Secret::getKey('imp'), Horde::getFormData('pass'));
+        $_SESSION['imp']['pass'] = Secret::write(Secret::getKey('imp'), Horde::getFormData('pass'));
 
         /* We might need to override some of the defaults with
            environmental settings. */
         include_once IMP_BASE . '/config/servers.php';
 
         if (Auth::getAuth()) {
-            $imp['uniquser'] = Auth::getAuth();
+            $_SESSION['imp']['uniquser'] = Auth::getAuth();
         } elseif (($conf['server']['server_list'] != 'none') && !empty($servers[$server]['realm'])) {
-            $imp['uniquser'] = $imp['user'] . '@' . $servers[$server]['realm'];
+            $_SESSION['imp']['uniquser'] = $_SESSION['imp']['user'] . '@' . $servers[$server]['realm'];
         } elseif (!empty($realm)) {
-            $imp['uniquser'] = $imp['user'] . '@' . $realm;
+            $_SESSION['imp']['uniquser'] = $_SESSION['imp']['user'] . '@' . $realm;
         } else {
-            $imp['uniquser'] = $imp['user'];
+            $_SESSION['imp']['uniquser'] = $_SESSION['imp']['user'];
         }
 
         if ($conf['server']['server_list'] != 'none'
             && isset($servers[$server])
             && is_array($servers[$server])) {
 
-            $imp['server'] = $servers[$server]['server'];
-            $imp['port'] = $servers[$server]['port'];
-            $imp['protocol'] = $servers[$server]['protocol'];
-            $imp['maildomain'] = $servers[$server]['maildomain'];
-            $imp['namespace'] = $servers[$server]['namespace'];
-            $imp['folders'] = $servers[$server]['folders'];
+            $_SESSION['imp']['server'] = $servers[$server]['server'];
+            $_SESSION['imp']['port'] = $servers[$server]['port'];
+            $_SESSION['imp']['protocol'] = $servers[$server]['protocol'];
+            $_SESSION['imp']['maildomain'] = $servers[$server]['maildomain'];
+            $_SESSION['imp']['namespace'] = $servers[$server]['namespace'];
+            $_SESSION['imp']['folders'] = $servers[$server]['folders'];
 
             if (($conf['mailer']['type'] == 'smtp') &&
                 !empty($servers[$server]['smtphost'])) {
-                $imp['smtphost'] = $servers[$server]['smtphost'];
+                $_SESSION['imp']['smtphost'] = $servers[$server]['smtphost'];
             }
         } else {
             foreach ($servers as $key => $curServer) {
@@ -98,50 +98,49 @@ class IMP {
                 }
             }
             if ($conf['server']['change_server']) {
-                $imp['server'] = $server;
+                $_SESSION['imp']['server'] = $server;
             } else {
-                $imp['server'] = $servers[$server_key]['server'];
+                $_SESSION['imp']['server'] = $servers[$server_key]['server'];
             }
             if ($conf['server']['change_port']) {
-                $imp['port'] = Horde::getFormData('port', '');
+                $_SESSION['imp']['port'] = Horde::getFormData('port', '');
             } else {
-                $imp['port'] = $servers[$server_key]['port'];
+                $_SESSION['imp']['port'] = $servers[$server_key]['port'];
             }
             if ($conf['server']['change_protocol']) {
-                $imp['protocol'] = Horde::getFormData('protocol', '');
+                $_SESSION['imp']['protocol'] = Horde::getFormData('protocol', '');
             } else {
-                $imp['protocol'] = $servers[$server_key]['protocol'];
+                $_SESSION['imp']['protocol'] = $servers[$server_key]['protocol'];
             }
             if ($conf['server']['change_folders']) {
-                $imp['folders'] = Horde::getFormData('folders');
+                $_SESSION['imp']['folders'] = Horde::getFormData('folders');
             } else {
-                $imp['folders'] = $servers[$server_key]['folders'];
+                $_SESSION['imp']['folders'] = $servers[$server_key]['folders'];
             }
-            $imp['maildomain'] = Horde::getFormData('maildomain', '');
-            $imp['namespace'] = Horde::getFormData('namespace', '');
+            $_SESSION['imp']['maildomain'] = Horde::getFormData('maildomain', '');
+            $_SESSION['imp']['namespace'] = Horde::getFormData('namespace', '');
         }
 
-        $protocols = explode('/', $imp['protocol']);
+        $protocols = explode('/', $_SESSION['imp']['protocol']);
         if (is_array($protocols)) {
-            $imp['base_protocol'] = $protocols[0];
+            $_SESSION['imp']['base_protocol'] = $protocols[0];
         } else {
-            $imp['base_protocol'] = $imp['protocol'];
+            $_SESSION['imp']['base_protocol'] = $_SESSION['imp']['protocol'];
         }
 
-        $imp['mailbox'] = '';
+        $_SESSION['imp']['mailbox'] = '';
         if (IMP::authenticate(OP_HALFOPEN) === true) {
             if ($registry->getMethod('auth/login') == 'imp') {
-                Auth::setAuth($imp['uniquser'], array('password' => $_POST['pass']));
+                Auth::setAuth($_SESSION['imp']['uniquser'], array('password' => Horde::dispelMagicQuotes($_POST['pass'])));
                 $registry->loadPrefs();
             }
             global $prefs;
 
             /* Set the session variables. These are cached. */
-            $imp['mailbox'] = $prefs->getValue('mailbox');
-            IMP::setLabel($imp);
+            $_SESSION['imp']['mailbox'] = $prefs->getValue('mailbox');
+            IMP::setLabel($_SESSION['imp']);
 
-            $_SESSION['imp'] = &$imp;
-            session_register('imp');
+            $GLOBALS['imp'] = &$_SESSION['imp'];
 
             return true;
         } else {
@@ -230,59 +229,59 @@ class IMP {
             }
         }
 
-        global $imp, $conf;
-        if (!(isset($imp) && is_array($imp))) {
+        global $conf;
+        if (!(isset($_SESSION['imp']) && is_array($_SESSION['imp']))) {
             if (isset($GLOBALS['prefs'])) {
                 $GLOBALS['prefs']->cleanup(true);
             }
             return 'session';
         }
 
-        switch ($imp['base_protocol']) {
+        switch ($_SESSION['imp']['base_protocol']) {
         case 'pop3':
-            $connstr = IMP::serverString($imp['protocol']);
+            $connstr = IMP::serverString($_SESSION['imp']['protocol']);
             $flags &= ~OP_ANONYMOUS;
             $flags &= ~OP_HALFOPEN;
-            $imp['thismailbox'] = '';
+            $_SESSION['imp']['thismailbox'] = '';
             break;
         default:
             $flags &= ~OP_ANONYMOUS;
-            $imp['thismailbox'] = Horde::getFormData('thismailbox', $imp['mailbox']);
-            if ($imp['thismailbox'] == '**search') {
+            $_SESSION['imp']['thismailbox'] = Horde::getFormData('thismailbox', $_SESSION['imp']['mailbox']);
+            if ($_SESSION['imp']['thismailbox'] == '**search') {
                 $aindex = Horde::getFormData('array_index');
-                if (($aindex !== null) && isset($imp['searchfolders'])) {
-                    $tmp = explode(':', $imp['messagefolders']);
-                    $connstr = IMP::serverString($imp['protocol']) . $tmp[$aindex];
+                if (($aindex !== null) && isset($_SESSION['imp']['searchfolders'])) {
+                    $tmp = explode(':', $_SESSION['imp']['messagefolders']);
+                    $connstr = IMP::serverString($_SESSION['imp']['protocol']) . $tmp[$aindex];
                 } else {
-                    $connstr = IMP::serverString($imp['protocol']);
+                    $connstr = IMP::serverString($_SESSION['imp']['protocol']);
                     $flags |= OP_HALFOPEN;
                 }
             } else {
-                $connstr = IMP::serverString($imp['protocol']) . $imp['thismailbox'];
+                $connstr = IMP::serverString($_SESSION['imp']['protocol']) . $_SESSION['imp']['thismailbox'];
             }
             break;
         }
 
         if ($flags == 0) {
-            $imp['stream'] = @imap_open($connstr, $imp['user'], Secret::read(Secret::getKey('imp'), $imp['pass']));
+            $_SESSION['imp']['stream'] = @imap_open($connstr, $_SESSION['imp']['user'], Secret::read(Secret::getKey('imp'), $_SESSION['imp']['pass']));
         } else {
-            $imp['stream'] = @imap_open($connstr, $imp['user'], Secret::read(Secret::getKey('imp'), $imp['pass']), $flags);
+            $_SESSION['imp']['stream'] = @imap_open($connstr, $_SESSION['imp']['user'], Secret::read(Secret::getKey('imp'), $_SESSION['imp']['pass']), $flags);
         }
 
-        if (!$imp['stream']) {
-            if (!empty($imp['server']) && !empty($imp['port']) &&
-                !empty($imp['protocol']) && !empty($imp['user'])) {
+        if (!$_SESSION['imp']['stream']) {
+            if (!empty($_SESSION['imp']['server']) && !empty($_SESSION['imp']['port']) &&
+                !empty($_SESSION['imp']['protocol']) && !empty($_SESSION['imp']['user'])) {
                 if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
                     $entry = sprintf('FAILED LOGIN %s (forwarded for [%s]) to %s:%s[%s] as %s',
-                                     $_SERVER['REMOTE_ADDR'], $_SERVER['HTTP_X_FORWARDED_FOR'], $imp['server'], $imp['port'], $imp['protocol'], $imp['user']);
+                                     $_SERVER['REMOTE_ADDR'], $_SERVER['HTTP_X_FORWARDED_FOR'], $_SESSION['imp']['server'], $_SESSION['imp']['port'], $_SESSION['imp']['protocol'], $_SESSION['imp']['user']);
                 } else {
                     $entry = sprintf('FAILED LOGIN %s to %s:%s[%s] as %s',
-                                     $_SERVER['REMOTE_ADDR'], $imp['server'], $imp['port'], $imp['protocol'], $imp['user']);
+                                     $_SERVER['REMOTE_ADDR'], $_SESSION['imp']['server'], $_SESSION['imp']['port'], $_SESSION['imp']['protocol'], $_SESSION['imp']['user']);
                 }
                 Horde::logMessage($entry, __FILE__, __LINE__, LOG_ERR);
             }
 
-            $imp = null;
+            $_SESSION['imp'] = null;
             session_unregister('imp');
             if (isset($GLOBALS['prefs'])) {
                 $GLOBALS['prefs']->cleanup(true);
@@ -316,9 +315,9 @@ class IMP {
 
     function loginTasks()
     {
-        global $actionID, $hordeMessageStack, $imp, $prefs;
+        global $actionID, $hordeMessageStack, $prefs;
 
-        if ($actionID == IMP_LOGIN || !empty($imp['_login'])) {
+        if ($actionID == IMP_LOGIN || !empty($_SESSION['imp']['_login'])) {
             /* Get UNIX timestamp of the last time the user logged in. */
             $last_login = $prefs->getValue('last_login');
 
@@ -349,7 +348,7 @@ class IMP {
             }
 
             /* If the user wants to run filters on login, make sure they get run. */
-            if ($prefs->getValue('filter_on_login') && $imp['mailbox'] == 'INBOX') {
+            if ($prefs->getValue('filter_on_login') && $_SESSION['imp']['mailbox'] == 'INBOX') {
                 $actionID = FILTER;
             }
 
@@ -361,7 +360,7 @@ class IMP {
                will be displayed on the initial login screen. */
             $_SESSION['logintasks'] = $hordeMessageStack;
 
-            $imp['_login'] = null;
+            $_SESSION['imp']['_login'] = null;
         }
     }
 
@@ -453,12 +452,10 @@ class IMP {
      */
     function serverString($protocol = null)
     {
-        global $imp;
-
         if (!isset($protocol)) {
-            $protocol = $imp['protocol'];
+            $protocol = $_SESSION['imp']['protocol'];
         }
-        return '{' . $imp['server'] . ':' . $imp['port'] . '/' . $protocol . '}';
+        return '{' . $_SESSION['imp']['server'] . ':' . $_SESSION['imp']['port'] . '/' . $protocol . '}';
     }
 
     /**
@@ -469,7 +466,7 @@ class IMP {
      *
      * @access private
      *
-     * @param array &$imp  The hash to pull mailbox information from and put
+     * @param array &$imp The hash to pull mailbox information from and put
      *                    the label in.
      */
     function setLabel(&$imp)
@@ -499,9 +496,9 @@ class IMP {
      *
      * @return string The folder, with any prefix gone.
      */
-    function displayFolder($folder, $decode = true) {
-        global $imp;
-        $prefix = IMP::serverString($imp['protocol']);
+    function displayFolder($folder, $decode = true)
+    {
+        $prefix = IMP::serverString($_SESSION['imp']['protocol']);
         if (substr($folder, 0, strlen($prefix)) == $prefix) {
             $folder = substr($folder, strlen($prefix));
         }
@@ -547,7 +544,7 @@ class IMP {
         /*
          * If both the mailbox and the host are empty, return an empty
          * string.  If we just let this case fall through, the call to
-         * imap_rfc822_write_address will end up return just a '@', which
+         * IMP::rfc822WriteAddress will end up return just a '@', which
          * is undesirable.
          */
         if (empty($ob->mailbox) && empty($ob->host)) {
@@ -573,7 +570,7 @@ class IMP {
         }
 
         /* Return the trimmed, formatted email address. */
-        return Mime::trimEmailAddress(imap_rfc822_write_address($ob->mailbox,
+        return Mime::trimEmailAddress(IMP::rfc822WriteAddress($ob->mailbox,
                                                                 $ob->host, $ob->personal));
     }
 
@@ -630,13 +627,17 @@ class IMP {
      */
     function bareAddress($address, $multiple = false)
     {
-        global $imp;
-
         $addressList = array();
 
-        $from = imap_rfc822_parse_adrlist($address, $imp['maildomain']);
+        $from = imap_rfc822_parse_adrlist($address, $_SESSION['imp']['maildomain']);
         foreach ($from as $entry) {
-            $addressList[] = $entry->mailbox . '@' . $entry->host;
+            if (isset($entry->mailbox)) {
+                if (isset($entry->host)) {
+                    $addressList[] = $entry->mailbox . '@' . $entry->host;
+                } else {
+                    $addressList[] = $entry->mailbox;
+                }
+            }
         }
 
         if ($multiple) {
@@ -661,9 +662,9 @@ class IMP {
      */
     function flist($filter = array())
     {
-        include_once IMP_BASE . '/lib/Folder.php';
-        global $imp, $conf, $prefs;
-        return IMP_Folder::flist($imp['stream'], IMP::serverString(), $imp['folders'], $prefs->getValue('subscribe'), $filter, $conf['server']['hierarchies'], $conf['server']['show_dotfiles'], $imp['namespace']);
+        require_once IMP_BASE . '/lib/Folder.php';
+        global $conf, $prefs;
+        return IMP_Folder::flist($_SESSION['imp']['stream'], IMP::serverString(), $_SESSION['imp']['folders'], $prefs->getValue('subscribe'), $filter, $conf['server']['hierarchies'], $conf['server']['show_dotfiles'], $_SESSION['imp']['namespace']);
     }
 
     /**
@@ -677,7 +678,7 @@ class IMP {
      * @param string $heading   (optional) An optional string to use as
      *                          the label for an empty-value option at
      *                          the top * of the list. Defaults to none.
-     * @param boolean $abbrev   (optional) If true, bbreviate long mailbox names
+     * @param boolean $abbrev   (optional) If true, abbreviate long mailbox names
      *                          by replacing the middle of the name with
      *                          '...'. Defaults to true.
      * @param array $filter     (optional) An array of mailboxes to ignore.
@@ -733,24 +734,24 @@ class IMP {
      */
     function flistUnsubscribed($filter = array())
     {
-        include_once IMP_BASE . '/lib/Folder.php';
-        global $imp, $conf;
-        return IMP_Folder::flistUnsubscribed($imp['stream'], IMP::serverString(), $imp['folders'], $filter, $conf['server']['hierarchies'], $conf['server']['show_dotfiles'], $imp['namespace']);
+        require_once IMP_BASE . '/lib/Folder.php';
+        global $conf;
+        return IMP_Folder::flistUnsubscribed($_SESSION['imp']['stream'], IMP::serverString(), $_SESSION['imp']['folders'], $filter, $conf['server']['hierarchies'], $conf['server']['show_dotfiles'], $_SESSION['imp']['namespace']);
     }
 
     function getThisMailbox()
     {
-        global $imp, $array_index;
+        global $array_index;
 
-        if ($imp['mailbox'] == '**search' && isset($array_index) &&
-            array_key_exists('messagefolders', $imp)) {
-            $tmp = explode(':', $imp['messagefolders']);
+        if ($_SESSION['imp']['mailbox'] == '**search' && isset($array_index) &&
+            array_key_exists('messagefolders', $_SESSION['imp'])) {
+            $tmp = explode(':', $_SESSION['imp']['messagefolders']);
             if (array_key_exists($array_index, $tmp)) {
                 return $tmp[$array_index];
             }
         }
 
-        return $imp['mailbox'];
+        return $_SESSION['imp']['mailbox'];
     }
 
     /**
@@ -885,35 +886,35 @@ class IMP {
      */
     function logoutUrl($uri, $reason = null)
     {
-        global $conf, $imp;
+        global $conf;
 
         if (!strstr($uri, '?')) {
             $uri .= '?1=1';
         }
 
-        if (!empty($imp['user'])) {
-            $uri .= '&imapuser=' . urlencode($imp['user']);
+        if (!empty($_SESSION['imp']['user'])) {
+            $uri .= '&imapuser=' . urlencode($_SESSION['imp']['user']);
         } elseif (Horde::getFormData('imapuser')) {
             $uri .= '&imapuser=' . urlencode(Horde::getFormData('imapuser'));
         }
 
         if (Horde::getFormData('server')) {
             $uri .= '&server=' . urlencode(Horde::getFormData('server'));
-        } elseif (!empty($imp['server'])) {
-            $uri .= '&server=' . urlencode($imp['server']);
+        } elseif (!empty($_SESSION['imp']['server'])) {
+            $uri .= '&server=' . urlencode($_SESSION['imp']['server']);
         }
 
-        if (!empty($imp['port'])) {
-            $uri .= '&port=' . urlencode($imp['port']);
+        if (!empty($_SESSION['imp']['port'])) {
+            $uri .= '&port=' . urlencode($_SESSION['imp']['port']);
         }
-        if (!empty($imp['protocol'])) {
-            $uri .= '&protocol=' . urlencode($imp['protocol']);
+        if (!empty($_SESSION['imp']['protocol'])) {
+            $uri .= '&protocol=' . urlencode($_SESSION['imp']['protocol']);
         }
-        if (!empty($imp['folders'])) {
-            $uri .= '&folders=' . urlencode($imp['folders']);
+        if (!empty($_SESSION['imp']['folders'])) {
+            $uri .= '&folders=' . urlencode($_SESSION['imp']['folders']);
         }
-        if (!empty($imp['language'])) {
-            $uri .= '&language=' . urlencode($imp['language']);
+        if (!empty($_SESSION['imp']['language'])) {
+            $uri .= '&language=' . urlencode($_SESSION['imp']['language']);
         }
         if (!empty($reason)) {
             $uri .= '&reason=' . urlencode($reason);
@@ -931,8 +932,7 @@ class IMP {
      */
     function getPart($mime)
     {
-        global $imp;
-        return imap_fetchbody($imp['stream'], $mime->index, $mime->imap_id, FT_UID);
+        return imap_fetchbody($_SESSION['imp']['stream'], $mime->index, $mime->imap_id, FT_UID);
     }
 
     /**
@@ -945,19 +945,19 @@ class IMP {
      */
     function getDecodedPart($mime)
     {
-        global $imp, $registry;
+        global $registry;
 
         if ($mime->encoding == ENCBASE64) {
-            $msg = imap_base64(imap_fetchbody($imp['stream'], $mime->index, $mime->imap_id, FT_UID));
+            $msg = imap_base64(imap_fetchbody($_SESSION['imp']['stream'], $mime->index, $mime->imap_id, FT_UID));
         } elseif ($mime->encoding == ENCQUOTEDPRINTABLE) {
-            $raw = imap_fetchbody($imp['stream'], $mime->index, $mime->imap_id, FT_UID);
+            $raw = imap_fetchbody($_SESSION['imp']['stream'], $mime->index, $mime->imap_id, FT_UID);
             $data = imap_qprint($raw);
             if (empty($data)) {
                 $data = $raw;
             }
             $msg = $data;
         } else {
-            $msg = imap_fetchbody($imp['stream'], $mime->index, $mime->imap_id, FT_UID);
+            $msg = imap_fetchbody($_SESSION['imp']['stream'], $mime->index, $mime->imap_id, FT_UID);
         }
 
         /* Convert Cyrillic character sets. */
@@ -1057,7 +1057,6 @@ class IMP {
             $row .= '<td>&nbsp;</td>';
         } else {
             require_once IMP_BASE . '/lib/Browser.php';
-            global $imp;
             $params = array();
             $params['thismailbox'] = urlencode(IMP::getThisMailbox());
             $params['index'] = $index;
@@ -1091,7 +1090,8 @@ class IMP {
             foreach ($lines as $line) {
                 $line = Text::expandEnvironment($line);
                 if (!empty($line)) {
-                    list($key, $val) = explode(':', $line);
+                    list($key, $val) = explode(':', $line, 2);
+                    $val = strtr($val, ':', '_');
                     $headers[$key] = trim($val);
                 }
             }
@@ -1108,7 +1108,7 @@ class IMP {
      */
     function prepareMailerParams()
     {
-        global $conf, $imp;
+        global $conf;
 
         /*
          * We don't actually want to alter the contents of the $conf['mailer']
@@ -1121,8 +1121,8 @@ class IMP {
          * Force the SMTP host value to the current SMTP server if one has been
          * selected for this connection.
          */
-         if (!empty($imp['smtphost'])) {
-             $params['host'] = $imp['smtphost'];
+         if (!empty($_SESSION['imp']['smtphost'])) {
+             $params['host'] = $_SESSION['imp']['smtphost'];
          }
 
         /*
@@ -1134,8 +1134,8 @@ class IMP {
          * well.
          */
         if (!empty($params['auth'])) {
-            $params['username'] = $imp['user'];
-            $params['password'] = Secret::read(Secret::getKey('imp'), $imp['pass']);
+            $params['username'] = $_SESSION['imp']['user'];
+            $params['password'] = Secret::read(Secret::getKey('imp'), $_SESSION['imp']['pass']);
         }
 
         return $params;
@@ -1179,24 +1179,27 @@ class IMP {
      */
     function filterMailbox()
     {
-        global $imp, $prefs;
+        global $prefs;
 
         $filters = @unserialize($prefs->getValue('filters'));
         if (is_array($filters) && count($filters) &&
-            !strstr($imp['protocol'], 'pop3')) {
+            !strstr($_SESSION['imp']['protocol'], 'pop3')) {
             include_once IMP_BASE . '/lib/Folder.php';
             $baseQuery = 'UNDELETED ';
             $applied = array();
             for ($i = 0; $i < count($filters); $i++) {
+                if (!isset($filters[$i])) {
+                    break;
+                }
                 for ($j = 0; $j < count($filters[$i]['fields']); $j++) {
                     $query = $baseQuery . strtoupper($filters[$i]['fields'][$j]) . ' "' . $filters[$i]['text'] . '"';
-                    $indices = @imap_search($imp['stream'], $query, SE_UID);
+                    $indices = @imap_search($_SESSION['imp']['stream'], $query, SE_UID);
                     if (isset($indices) && is_array($indices)) {
                         $indices = array_diff($indices, $applied);
                         if (count($indices)) {
                             if ($filters[$i]['action'] == 'delete') {
                                 if ($prefs->getValue('show_filter_msg')) {
-                                    $overview = @imap_fetch_overview($imp['stream'], implode(',', $indices), FT_UID);
+                                    $overview = @imap_fetch_overview($_SESSION['imp']['stream'], implode(',', $indices), FT_UID);
                                     foreach ($overview as $message) {
                                         if ($prefs->getValue('use_trash')) {
                                             Horde::raiseMessage(sprintf(_("Filter Activity: The message \"%s\" from \"%s\" has been moved to your Trash folder."), isset($message->subject) ? MIME::decode($message->subject) : _("[No Subject]"), MIME::decode($message->from)), HORDE_MESSAGE);
@@ -1207,10 +1210,10 @@ class IMP {
                                 }
                                 IMP_Message::delete($indices);
                                 $applied = array_merge($applied, $indices);
-                            } elseif ($filters[$i]['folder'] != $imp['mailbox']) {
-                                if (IMP_Folder::exists($imp['stream'], $filters[$i]['folder'])) {
+                            } elseif ($filters[$i]['folder'] != $_SESSION['imp']['mailbox']) {
+                                if (IMP_Folder::exists($_SESSION['imp']['stream'], $filters[$i]['folder'])) {
                                     if ($prefs->getValue('show_filter_msg')) {
-                                        $overview = @imap_fetch_overview($imp['stream'], implode(',', $indices), FT_UID);
+                                        $overview = @imap_fetch_overview($_SESSION['imp']['stream'], implode(',', $indices), FT_UID);
                                         foreach ($overview as $message) {
                                             Horde::raiseMessage(sprintf(_("Filter Activity: The message \"%s\" from \"%s\" has been moved to the folder \"%s\"."), isset($message->subject) ? MIME::decode($message->subject) : _("[No Subject]"), MIME::decode($message->from), $filters[$i]['folder']), HORDE_MESSAGE);
                                         }
@@ -1328,4 +1331,66 @@ class IMP {
         return $output;
     }
 
+    /**
+     * Quotes and escapes the given string if necessary.
+     *
+     * @access private
+     *
+     * @param string $str              The string to be quoted and escaped.
+     * @param optional string $filter  A list of characters that make it
+     *                                 necessary to quote the string if they
+     *                                 occur.
+     *
+     * @return string  The correctly quoted and escaped string.
+     *
+     * @since IMP 3.2.2
+     */
+    function _rfc822Encode($str, $filter = '')
+    {
+        $filter .= "()<>@,;:\\\"[]\1\2\3\4\5\6\7\10\11\12\13\14\15\16\17\20\21\22\23\24\25\26\27\30\31\32\33\34\35\36\37\177";
+
+        if (strcspn($str, $filter) != strlen($str)) {
+            return '"' . str_replace('"', '\\"', str_replace('\\', '\\\\', $str)) . '"';
+        } else {
+            return $str;
+        }
+    }
+
+    /**
+     * Builds an RFC 822 compliant email address.
+     *
+     * @access public
+     *
+     * @param string $mailbox            Mailbox name.
+     * @param optional string $host      Domain name of mailbox's host.
+     * @param optional string $personal  Personal name phrase.
+     *
+     * @return string  The correctly escaped and quoted
+     *                 "$personal <$mailbox@$host>" string.
+     *
+     * @since IMP 3.2.2
+     */
+    function rfc822WriteAddress($mailbox, $host = null, $personal = '')
+    {
+        $address = '';
+
+        if (!empty($personal)) {
+            $address .= IMP::_rfc822Encode($personal, '.');
+            $address .= ' <';
+        }
+
+        if (!is_null($host)) {
+            $address .= IMP::_rfc822Encode($mailbox);
+            if (substr($host, 0, 1) != '@') {
+                $address .= '@' . $host;
+            }
+        }
+
+        if (!empty($personal)) {
+            $address .= '>';
+        }
+
+        return $address;
+    }
+
 }
