@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: imp/mailbox.php,v 2.296.2.55 2003/03/11 03:28:27 slusarz Exp $
+ * $Horde: imp/mailbox.php,v 2.296.2.61 2003/08/07 09:32:11 jan Exp $
  *
  * Copyright 1999-2003 Charles J. Hagenbuch <chuck@horde.org>
  * Copyright 1999-2003 Jon Parise <jon@horde.org>
@@ -89,9 +89,17 @@ require_once IMP_BASE . '/lib/Identity/IMP.php';
 /* Initialize $actionID to a resonable value if it hasn't already been set. */
 $actionID = Horde::getFormData('actionID');
 
+if (empty($notification)) {
+    require_once HORDE_BASE . '/lib/Notification.php';
+    $notification = &Notification::singleton();
+    $notification->attach('status');
+}
+$show_whitelist = $registry->hasMethod('filter/whitelistFrom');
+
 IMP::checkAuthentication();
 
-$newmail_popup = (Horde::getFormData('newmail_popup') == 'no') ? false : true;
+$newmail_popup = ($prefs->getValue('nav_popup') &&
+                  (Horde::getFormData('newmail_popup') != 'no'));
 
 if (($actionID == IMP_LOGIN) || ($actionID == LOGIN_COMPOSE)) {
     if (isset($new_lang)) {
@@ -136,19 +144,47 @@ $identity = new Identity_IMP();
 switch ($actionID) {
 
  case IMP_BLACKLIST:
-     $filters = @unserialize($prefs->getValue('filters'));
-     foreach ($indices as $msgnum) {
-         $h = @imap_header($imp['stream'], imap_msgno($imp['stream'], $msgnum));
-         if (isset($h->from[0])) {
-             $ob = $h->from[0];
-             $addr = Mime::trimEmailAddress(imap_rfc822_write_address($ob->mailbox, $ob->host, ''));
-             $filters[] = array('fields' => array('from'), 'text' => chop(trim($addr)), 'action' => 'delete');
+     if ($registry->hasMethod('filter/blacklistFrom')) {
+         $addresses = array();
+         foreach ($indices as $msgnum) {
+             $h = @imap_header($imp['stream'], imap_msgno($imp['stream'], $msgnum));
+             if (isset($h->from[0])) {
+                 $ob = $h->from[0];
+                 $addr = Mime::trimEmailAddress(IMP::rfc822WriteAddress($ob->mailbox, $ob->host, ''));
+                 $addresses[] = chop(trim($addr));
+             }
          }
+         $registry->call('filter/blacklistFrom', array($addresses));
+     } else {
+         $filters = @unserialize($prefs->getValue('filters'));
+         foreach ($indices as $msgnum) {
+             $h = @imap_header($imp['stream'], imap_msgno($imp['stream'], $msgnum));
+             if (isset($h->from[0])) {
+                 $ob = $h->from[0];
+                 $addr = Mime::trimEmailAddress(IMP::rfc822WriteAddress($ob->mailbox, $ob->host, ''));
+                 $filters[] = array('fields' => array('from'), 'text' => chop(trim($addr)), 'action' => 'delete');
+             }
+         }
+         $prefs->setValue('filters', serialize($filters));
+         $prefs->store();
+         header('Location: ' . Horde::applicationUrl('filters.php'));
+         exit;
+     }
+     break;
+
+ case IMP_WHITELIST:
+     if ($show_whitelist) {
+         $addresses = array();
+         foreach ($indices as $msgnum) {
+             $h = @imap_header($imp['stream'], imap_msgno($imp['stream'], $msgnum));
+             if (isset($h->from[0])) {
+                 $ob = $h->from[0];
+                 $addr = Mime::trimEmailAddress(IMP::rfc822WriteAddress($ob->mailbox, $ob->host, ''));
+                 $addresses[] = chop(trim($addr));
+             }
+         }
+         $registry->call('filter/whitelistFrom', array($addresses));
      }
-     $prefs->setValue('filters', serialize($filters));
-     $prefs->store();
-     header('Location: ' . Horde::applicationUrl('filters.php'));
-     exit;
      break;
 
  case MESSAGE_MISSING:
@@ -340,7 +376,7 @@ switch ($actionID) {
          }
      }
      if ($url = Horde::getFormData('return_url')) {
-         header('Location: ' . $url);
+         header('Location: ' . str_replace('&amp;', '&', $url));
          exit;
      }
      break;
@@ -358,14 +394,14 @@ if ($imp['mailbox'] == '**search') {
     if ($prefs->getValue('delhide')) {
         $msgs = imap_search($imp['stream'], 'UNDELETED');
         $imp['msgcount'] = ($msgs !== false) ? count($msgs) : 0;
-        if ($prefs->getValue('nav_popup') && $newmail_popup) {
+        if ($newmail_popup) {
             $new = imap_search($imp['stream'], 'RECENT UNDELETED');
             $newmsgs = $new ? count($new) : 0;
         }
     } else {
         $check = imap_check($imp['stream']);
         $imp['msgcount'] = (is_object($check) && isset($check->Nmsgs)) ? $check->Nmsgs : 0;
-        if ($prefs->getValue('nav_popup') && $newmail_popup) {
+        if ($newmail_popup) {
             $new = imap_search($imp['stream'], 'RECENT');
             $newmsgs = $new ? count($new) : 0;
         }
@@ -545,6 +581,7 @@ if ($browser->hasFeature('javascript')) {
 }
 require IMP_BASE . '/menu.php';
 require IMP_BASE . '/status.php';
+$notification->notify();
 
 if (!empty($conf['hooks']['quota']) && function_exists($conf['hooks']['quota'])) {
     echo call_user_func($conf['hooks']['quota'], $imp);
@@ -840,7 +877,7 @@ $navform = 2;
 require IMP_TEMPLATES . '/mailbox/navbar.inc';
 require IMP_TEMPLATES . '/mailbox/footer.inc';
 
-if ($newmail_popup && $prefs->getValue('nav_popup') && ($newmsgs > 0)) {
+if ($newmail_popup && ($newmsgs > 0)) {
     include IMP_TEMPLATES . '/mailbox/alert.inc';
 }
 
